@@ -36,15 +36,13 @@ use Log::Log4perl qw(get_logger);
 use File::Temp qw(tempfile);
 use Grid::Request::Param;
 
-my $DEFAULT_BLOCK_SIZE = 100;
-
 my $logger = get_logger(__PACKAGE__);
 
 use vars qw( %VALID_OS %VALID_PARAM_ARGS %VALID_STATE
              %VALID_TYPE %VALID_CMD_TYPE );
 
 # The IO:Scalar and XML::Writer are pulled in with "require" if necessary.
-our $VERSION = '0.12';
+our $VERSION = qw$Revision: 8365 $[1];
 
 # Get rid of warnings about single usage.
 if ($^W) {
@@ -141,15 +139,21 @@ sub _init {
     # uid's in order to submit jobs on behalf of users.
     # We should use the real user id.
     $self->{username} = getpwuid($<);
-    $logger->debug("Set the username to the effective UID: " . $self->{username});
+    $logger->debug("Set the username according to effective uid: ",
+                   "$self->{username}.");
 
-    # Set the default block size.
-    $logger->debug("Setting the default block size.");
-    $self->{block_size} = $DEFAULT_BLOCK_SIZE;
-    
-    foreach my $method (qw(block_size command class error getenv initialdir
-                           input output name project priority times evictable
-                           length runtime hosts)) {
+    $logger->debug("Setting the project.");
+    if ( exists($args{project}) && defined($args{project}) ) {
+        $self->project($args{project});
+    } else {
+        my $msg = "Mandatory 'project' attribute not provided.";
+        $logger->fatal($msg);
+        Grid::Request::Exception->throw($msg);
+    }
+
+    foreach my $method qw(command class error getenv initialdir input output
+                          name priority times evictable length runtime hosts
+                         ) {
         if (exists($args{$method}) && defined($args{$method})) {
             $logger->info("Initializing $method.");
             $self->$method( $args{$method} );
@@ -186,63 +190,6 @@ sub account {
 }
 
 
-=item $obj->block_size( [ $scalar | $code_ref ] );
-
-B<Description:> By default, Master/Worker jobs have a default block size of
-100.  That is to say, that each worker on the grid will process 100 elements of
-the overall pool of job invocations. However, this isn't always appropriate.
-The user may override the default block size by calling this method and setting
-the block size to an alternate value (a positive integer). The user may also
-provide an anonoymous subroutine (code reference) so that the block size can be
-computed dynamically. If choosing to pass a subroutine , the code will be passed
-two arguments: the command that will be invoked, and the number of elements that
-will be iterated over. The subroutine can then use these pieces of information
-to compute the block size. The subroutine MUST return a positive integer scalar
-or an exception will be thrown.
-
-B<Parameters:> A positive integer scalar, or an anonymous subroutine/code
-reference.
-
-B<Returns:> The block size scalar or code reference if called as an accessor
-(no-arguments). If the block size has not been explicitly set, then the default
-block size is returned. No return if called as a mutator.
-
-=cut
-
-sub block_size {
-    $logger->debug("In block_size");
-    my ($self, $block_size, @args) = @_;
-
-    if (scalar @args) {
-        Grid::Request::InvalidArgumentException->throw(
-            "Too many arguments passed to the 'block_size' method.");
-    }
-
-    if (defined $block_size) {
-        if (ref $block_size eq "CODE") {
-            $logger->debug("Got a code reference for the block size.");
-        } elsif (ref \$block_size eq "SCALAR") {
-            if ($block_size =~ /^-?\d+$/) {
-                if ($block_size <= 0) {
-                    Grid::Request::InvalidArgumentException->throw(
-                        "Scalar argument for 'block_size' must be a positive integer.");
-                } 
-            }
-        } else {
-            Grid::Request::InvalidArgumentException->throw(
-                "Illegal argument for 'block_size' method.");
-        }
-        
-        # If we get here, then we have an acceptable block size
-        $self->{block_size} = $block_size;
-    } elsif (exists($self->{block_size}) && defined($self->{block_size})) {
-        return $self->{block_size};
-    } else {
-        return undef;
-    }
-}
-
-
 =item $obj->add_param(@param_args);
 
 B<Description:> This is documented in Grid::Request (Request.pm).
@@ -303,7 +250,7 @@ sub add_param {
             # Make the type case insensitive from the perspective of the user
             # by always capitalizing what they passed in.
             $param_obj->type(uc($args[2]));
-        } elsif (@args > 3 || @args == 0 || @args == 2) {
+        } elsif (@args > 3 || @args == 0) {
             # This method was called incorrectly.
             $msg = "add_param() only accepts 1 or 3 arguments.";
             $logger->error($msg);
@@ -478,13 +425,15 @@ sub email {
 =item $obj->end_time()
 
 B<Description:> This method is used as a getter for the finish time of the
-command. If any other package (including main) attempts to set the end_time
-attribute with this method, an error will result.
+command. It may only be used as a setter by the ProxyServer module. If any
+other package (including main) attempts to set the end_time attribute with
+this method, an error will result.
 
 B<Parameters:> None.
 
-B<Returns:> The ending time of the command or undef if the end_time has not yet
-been established.
+B<Returns:> The ending time of the command (the time the DCE finished
+processing the command), or undef if the end_time has not yet been
+established.
 
 =cut
 
@@ -674,6 +623,37 @@ sub initialdir {
     }
 }
 
+
+=item $obj->length([length]);
+
+B<Description:> This method is used to characterize how long the request
+is expected to take to complete. For long running requests, an attempt to
+match appropriate resources is made. If unsure, leave this setting unset.
+
+B<Parameters:> "short", "medium", "long". No attempt is made to validate
+the length passed in when used as a setter.
+
+B<Returns:> The currently set length attribute (when called with no
+arguments).
+
+=cut
+
+sub length {
+    $logger->debug("In length.");
+    my ($self, $length, @args) = @_;
+
+    if (defined $length) {
+        $logger->warn("The length method takes only one argument ",
+                      "when making an assignment.") if @args;
+        $self->{length} = $length;
+    } elsif (exists($self->{length}) && defined($self->{length})) {
+        return $self->{length};
+    } else {
+        return undef;
+    }
+}
+
+
 =item $obj->name([name]);
 
 B<Description:> The name attribute for command objects is optional and is
@@ -773,7 +753,6 @@ sub cmd_type {
 }
 
 
-# Documeted in Grid::Request
 sub hosts {
     $logger->debug("In hosts.");
     my ($self, $hosts, @args) = @_;
@@ -788,7 +767,6 @@ sub hosts {
     }
 }
 
-# Documeted in Grid::Request
 sub pass_through {
     $logger->debug("In pass_through");
     my ($self, $pass_through, @args) = @_;
@@ -805,7 +783,6 @@ sub pass_through {
 }
 
 
-# Documeted in Grid::Request
 sub memory {
     $logger->debug("In memory");
     my ($self, $memory, @args) = @_;
@@ -934,8 +911,9 @@ sub priority {
 =item $obj->start_time([time]);
 
 B<Description:> This method is only intended to be used as a getter by end
-users. If any attempt is made to set the start_time attribute of the command
-object elsewhere, an error will occur.
+users. It may only be used as a setter by the ProxyServer module. If any
+attempt is made to set the start_time attribute of the command object
+elsewhere, an error will occur.
 
 B<Parameters:> None.
 
@@ -963,8 +941,9 @@ sub start_time {
 =item $obj->state([state]);
 
 B<Description:> This method is only intended to be used as a getter by end
-users. If any attempt is made to set the "state" attribute of the command
-object elsewhere, an error will occur. Valid states are:
+users. It may only be used as a setter by the ProxyServer module. If any
+attempt is made to set the "state" attribute of the command object
+elsewhere, an error will occur. Valid states are:
 
     INIT
     INTERRUPTED
@@ -1063,7 +1042,6 @@ sub times {
     if (defined $times) {
         $logger->warn("The times method takes only one argument ",
                       "when making an assignment.") if @args;
-        # TODO: Not robust enough of a check
         if ($times =~ m/\D/) {
             $logger->error("Encountered non-numeric 'times' attribute.");
             return undef;
@@ -1166,6 +1144,7 @@ sub to_xml {
     my $output   = $self->output;
     my $priority = $self->priority;
     my $times    = $self->times;
+    my $length   = $self->length;
     my $getenv   = $self->getenv;
     my $email    = $self->email;
     my @ids      = $self->ids();
@@ -1183,44 +1162,47 @@ sub to_xml {
                            );
 
     $w->startTag('command', 'type' => $cmd_type);
-    $w->dataElement('executable', $exe) if (defined($exe) && ($exe ne ""));
-    $w->dataElement('project', $project) if (defined($project) && ($project ne ""));
+    $w->dataElement('executable', $exe);
+    $w->dataElement('project', $project );
+    $w->dataElement('username', getpwuid($<));
     foreach my $id (@ids) { 
-        $w->dataElement('id', $id);
+        $w->dataElement('id', $id );
     }
     $w->startTag('config');
     
     $logger->debug("XML :  Opsys is $opsys \n");
-    $w->dataElement("opSys", $opsys) if (defined($opsys) && ($opsys ne ""));
-    $w->dataElement("class", $class) if (defined($class) && ($class ne ""));
+    $w->dataElement("opSys", $opsys) if (defined $opsys);
+    $w->dataElement("class", $class) if (defined($class));
     $w->dataElement("hosts", $hosts) if (defined($hosts) && ($hosts ne ""));
-    $w->dataElement("memory", $memory) if (defined($memory) && ($memory ne ""));
+    $w->dataElement("memory", $memory) if defined($memory);
     $w->dataElement("passThrough", $pass_through) if (defined($pass_through) && ($pass_through ne ""));
 
     # getenv and length are are not mandatory in the config block.
     # Check if they are defined before writing any XML for them.
     $w->dataElement('getenv', $getenv) if defined($getenv);
-    $w->dataElement('evictable', $evictable) if (defined($evictable) && ($evictable ne ""));
-    $w->dataElement('runningTime', $runtime) if (defined($runtime) && ($runtime ne ""));
+    $w->dataElement('length', $length) if (defined($length) && ($length ne ""));
+    $w->dataElement('evictable', $evictable) if (defined($evictable));
+    $w->dataElement('runningTime', $runtime) if (defined($runtime));
 					   
     $w->endTag('config');
 
+
     # Several fields are are not required. Check if they are defined
     # before writing any XML for them.
     $w->dataElement('email', $email) if (defined($email) && ($email ne ""));
-    $w->dataElement('name', $name) if (defined($name) && ($name ne ""));
-    $w->dataElement('times', $times) if (defined($times) && ($times ne ""));
+    $w->dataElement('name', $name) if defined($name);
+    $w->dataElement('times', $times) if defined($times);
     
     # Command Param block.
     my @params = $self->params;
     if (@params) {
         $logger->info("Coding params into XML document.");
-        foreach my $param (@params) {
-            $w->startTag('param', 'type' => $param->type );
-            if ( defined($param->key) && length($param->key) ) {
-                $w->dataElement('key', $param->key);
+        foreach my $ref (@params) {
+            $w->startTag('param', 'type' => $ref->{type} );
+            if ( exists($ref->{key}) && defined($ref->{key}) ) {
+                $w->dataElement('key', $ref->{key});
             }
-            $w->dataElement('value', $param->value) if (defined($param->value));
+            $w->dataElement('value', $ref->{value});
             $w->endTag('param');
         }
     } else {
@@ -1244,8 +1226,8 @@ sub to_xml {
 sub _write_temp_array_file {
     $logger->debug("In _write_temp_array_file.");
     my ($self, $arrayref) = @_;
-    # To aid in tracking down problems, we are going to use the hostname of
-    # the machine making the requests to name our temporary file.
+    # To aid in tracking down problems, we are going to use the hostname to
+    # of the machine making the requests to name our temporary file.
     require Sys::Hostname;
     # Get the temporary file location. This was probably set by the enclosing
     # Request object when add_param was invoked.
