@@ -26,7 +26,7 @@ use File::Which;
 use Log::Log4perl qw(:easy :levels);
 
 my $logger = get_logger(__PACKAGE__);
-our $config_section;
+our ($config_section, $drm_param);
 
 my $worker_name = "grid_request_worker";
 our $WORKER = which($worker_name);
@@ -35,7 +35,7 @@ if (! defined $WORKER) {
 }
 
 use vars qw($config $client $server);
-our $VERSION = '0.12';
+our $VERSION = do { my @r=(q$Revision: 8365 $=~/\d+/g); sprintf "%d."."%03d"x$#r,@r };
 
 if ($^W) {
     $VERSION = $VERSION;
@@ -43,6 +43,7 @@ if ($^W) {
 
 BEGIN {
     $config_section = "request";
+    $drm_param = "drm";
 
     my $central_config = "$ENV{HOME}/.grid_request.conf";
 
@@ -54,6 +55,11 @@ BEGIN {
             warn "Is it a valid INI file with a [" . $config_section . "] section?\n";
             exit 1;
         }
+        my $drm = $cfg->val($config_section, $drm_param);
+        if (! defined $drm) {
+            warn "The config file does not define a '" . $drm_param . "' parameter.\n";
+            exit 1;
+        }
     } else {
         warn "The config file $config does not exist or isn't readable.\n";
         exit 1;
@@ -165,6 +171,30 @@ sub debug {
     }
 }
 
+=item $obj->project([$project]);
+
+B<Description:> Used to set or get the project that the job
+is to be associated with. The project is necessary to track usage
+patterns of the infrastructure and to provide useful management
+information.
+
+B<Parameters:> $project, optional project argument.
+
+B<Returns:> $project, the currently set project.
+
+=cut
+
+sub project {
+    $logger->debug("In project.");
+    my ($self, @args) = @_;
+    if (scalar(@args)) {
+        $self->{project} = $args[0];
+    } else {
+       return $self->{project};
+    }
+}
+
+
 1;
 
 __END__
